package cn.com.qmth.stmms.biz.file.store.impl;

import cn.com.qmth.stmms.biz.file.store.FileStore;
import com.aliyun.oss.common.utils.BinaryUtil;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

/**
 * 本地磁盘文件管理工具
 */
public class DiskStore implements FileStore {

    private static final Logger log = LoggerFactory.getLogger(DiskStore.class);

    private File rootDir;

    public DiskStore(String path) {
        rootDir = new File(path);
        if (!rootDir.exists() && !rootDir.mkdirs()) {
            //自动创建目录失败
            throw new IllegalArgumentException("[file.store]" + path + ": auto mkdir faile");
        } else if (rootDir.isFile()) {
            //判断是否不是目录
            throw new IllegalArgumentException("[file.store]" + path + ": is a file");
        }
    }

    @Override
    public boolean write(String path, InputStream ins, String md5) {
        FileOutputStream ous = null;
        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
        try {
            IOUtils.copy(ins, buffer);
            byte[] data = buffer.toByteArray();
            if (!BinaryUtil.encodeMD5(data).equalsIgnoreCase(md5)) {
                throw new RuntimeException("md5 validate faile");
            }
            File target = new File(rootDir, path);
            target.getParentFile().mkdirs();
            ous = new FileOutputStream(target);
            ous.write(data);
            ous.close();
            return true;
        } catch (Exception e) {
            log.error("write file error:" + path, e);
            return false;
        } finally {
            IOUtils.closeQuietly(ous);
            IOUtils.closeQuietly(buffer);
        }
    }

    @Override
    public byte[] read(String path) {
        File file = new File(rootDir, path);
        if (file.exists() && file.isFile()) {
            FileInputStream ins = null;
            FileChannel channel = null;
            try {
                ins = new FileInputStream(file);
                channel = ins.getChannel();
                ByteBuffer byteBuffer = ByteBuffer.allocate((int) channel.size());
                while ((channel.read(byteBuffer)) > 0) {
                    //
                }
                return byteBuffer.array();
            } catch (Exception e) {
                log.error("read file error:" + path, e);
                return null;
            } finally {
                IOUtils.closeQuietly(ins);
                IOUtils.closeQuietly(channel);
            }
        } else {
            return null;
        }
    }

}
